unit MarcoWebComps;

interface

uses
  WebComp, DB, HTTPProd, SiteComp, WebForm, MidItems, SysUtils,
  Classes, AdaptReq, WebCntxt;

type
  TMarcoAdapterFormLayout = class(TAdapterFormLayout)
    function ImplLayoutLabelAndField(const HTMLLabel, HTMLField: string; Attributes: TLayoutAttributes): string; override;
  end;

  TMarcoAdapterFieldGroup = class (TAdapterFieldGroup)
  private
    FColumns: Integer;
    function FormatFields(ParentLayout: TLayout;
      Options: TWebContentOptions): string;
    procedure SetColumns(const Value: Integer);
  public
    function ImplContent(Options: TWebContentOptions; ParentLayout: TLayout): string; override;
  published
    property Columns: Integer read FColumns write SetColumns;
  end;

  TMarcoAdapterDisplayField = class (TAdapterDisplayField)
  private
    FColSpan: Integer;
    procedure SetColSpan(const Value: Integer);
  protected
    function GetLayoutAttributes: TLayoutAttributes; override;
  published
    property ColSpan: Integer read FColSpan write SetColSpan;
  end;

procedure Register;

implementation

{ TMarcoAdapterFormLayout }

function TMarcoAdapterFormLayout.ImplLayoutLabelAndField(const HTMLLabel,
  HTMLField: string; Attributes: TLayoutAttributes): string;
var
  cells: Integer;
begin
  if HTMLField = '' then
  begin
    Result := '';
    Exit;
  end;
  if Assigned(Attributes) then
  begin
    // done to bypass the fact that the StartFields
    // parameter is actually ignored!
    cells := 2;
    case Attributes.LabelPosition of
      lposLeft:
      begin
        Result := StartFields(cells) +
           Format('%0:s>%1:s %2:s</td>'#13#10,
            [Attributes.ControlAttributes, HTMLLabel, HTMLField]);
      end;
      lposRight:
      begin
        Result := '<b>not supported</b>';
      end;
      lposAbove:
        Result :=
          StartFields(cells) +
           Format('%0:s>%1:s <br> %2:s</td>'#13#10,
            [Attributes.ControlAttributes, HTMLLabel, HTMLField]);
      lposBelow:
        Result := '<b>not supported</b>';
    else
      Assert(False, 'Unknown position');
    end;

    // take into account colspan: move ahead one more column!
    cells := 1;
    if Pos ('colspan=2', Attributes.ControlAttributes) > 0 then
      NextColumn(1);
  end
  else
    Result :=
        StartFields(2) + Format('>%0:s</td>'#13#10'<td>%1:s</td>'#13#10,
          [HTMLLabel, HTMLField]);
end;

{ TMarcoAdapterForm }

const
  sSetAdapterMode = '<%%'#13#10 +
                     '{'#13#10 +                            // Do not localize
                     '   var lastMode=%0:s.Mode'#13#10 +         // Do not localize
                     '   if(lastMode=="")'#13#10 +     // Do not localize
                     '   {'#13#10 +         // Do not localize
                     '     %0:s.Mode=''%1:s'''#13#10 +          // Do not localize
                     '   } %%>'#13#10 +                          // Do not localize
                     '%2:s'#13#10 +                           // Do not localize
                    '<%% %0:s.Mode=lastMode'#13#10 +         // Do not localize
                     '}  %%>'#13#10;                           // Do not localize

const
  // Params 0 - AdapterName, 1 - MemberName, 2 - Indent
  sWriteHiddenFields =
  '<%%%2:s if (%0:s.%1:s != null)'#13#10 +
   '%2:s   {'#13#10 +
   '%2:s     %0:s.%1:s.WriteFields(Response)'#13#10 +
   '%2:s   } %%>'#13#10;
  sHiddenFields = 'HiddenFields';
  sHiddenRecordFields = 'HiddenRecordFields';
  // Params 0 - AdapterName, 1 - Row
  sJSGridForEach =
  '<%% {'#13#10 +
   '   var ri = 0'#13#10 +
   '   var e = new Enumerator(%0:s.Records)'#13#10 +
   '   for (; !e.atEnd(); e.moveNext())'#13#10 +
   '   { %%>'#13#10 +
   '%1:s' +
  '<%%  ri++'#13#10 +
   '   }'#13#10 +
   '   e.moveFirst()'#13#10 +
   '   } %%>'#13#10;

function TMarcoAdapterFieldGroup.ImplContent(Options: TWebContentOptions;
  ParentLayout: TLayout): string;
begin
  Result := FormatFields(ParentLayout, Options);
  if (Adapter <> nil) and (AdapterMode <> '') then
    Result := Format(sSetAdapterMode,
      [MakeAdapterVariableName(Self), AdapterMode, Result]);
end;

function TMarcoAdapterFieldGroup.FormatFields(ParentLayout: TLayout;
  Options: TWebContentOptions): string;
var
  FFormLayout: TAdapterFormLayout;
  I: Integer;
  Intf: ILayoutWebContent;
  VarName: string;
  A: TLayoutAttributes;

  function FormatField(Field: TComponent): string;
  var
    Intf: IWebContent;
  begin
    if Supports(IInterface(Field), IWebContent, Intf) then
      Result := Intf.Content(Options, FFormLayout)
    else
      Result := '';
  end;

  function FormatHiddenFields(const AdapterVarName: string): string;
  begin
    if AdapterVarName <> '' then
    begin
      Result := Format(sWriteHiddenFields, [AdapterVarName, sHiddenFields, '']) +
      Format(sWriteHiddenFields, [AdapterVarName, sHiddenRecordFields, '']);
    end;
  end;

begin
  FFormLayout := TMarcoAdapterFormLayout.Create(ParentLayout);
  if Columns > 0 then // FIX
    FFormLayout.ColumnCount := Columns;
  try
    FFormLayout.TableHeader := Format('<table%s>', [GetLayoutAttributes.ControlAttributes]);
    Result := '';
    for I := 0 to VisibleFields.Count - 1 do
      Result := Result + FormatField(VisibleFields[I]);
    VarName := MakeAdapterVariableName(Self);
    Result := Result + FFormLayout.EndLayout;
    if Supports(ParentLayout, ILayoutWebContent, Intf) then
      if ParentLayout is TAdapterFormLayout then
        Result := Intf.LayoutTable(Result, nil)
      else
      begin
        A := TLayoutAttributes.Create;
        try
          with A do
          begin
            AddQuotedAttrib(ControlAttributes, sAlignAttr, HTMLAlignValues[haCenter]);
            AddQuotedAttrib(ControlAttributes, sVAlignAttr, HTMLVAlignValues[haTop]);
            // Work better with a layout group
            Result := Intf.LayoutTable(#13#10 + Result, A);
          end
        finally
          A.Free;
        end;
      end;
    Result := Result + FormatHiddenFields(VarName);
  finally
    FreeAndNil(FFormLayout);
  end;
end;

procedure Register;
begin
  RegisterWebComponents([TMarcoAdapterFieldGroup, TMarcoAdapterDisplayField]);
end;

procedure TMarcoAdapterFieldGroup.SetColumns(const Value: Integer);
begin
  FColumns := Value;
end;

{ TMarcoAdapterDisplayField }

function TMarcoAdapterDisplayField.GetLayoutAttributes: TLayoutAttributes;
begin
  Result := inherited GetLayoutAttributes;
  Result.ControlAttributes := Result.ControlAttributes + ' colspan=' +
    IntToStr (FColSpan);
end;

procedure TMarcoAdapterDisplayField.SetColSpan(const Value: Integer);
begin
  FColSpan := Value;
end;

initialization

finalization
  UnRegisterWebComponents([TMarcoAdapterFieldGroup, TMarcoAdapterDisplayField]);

end.
