unit SimpleWebComps;

interface

uses
  WebComp, DB, HTTPProd, SiteComp, WebForm, MidItems, SysUtils,
  Classes, AdaptReq, WebCntxt, SiteConst;

type
  ISimpleProduceHtml = interface
    ['{98202025-053F-4D11-9DF3-244DC1C9A06A}']
    function GetHtml (Options: TWebContentOptions): string;
    function GetCols: Integer;
  end;

  TSimpleAdapterFieldGroup = class (TAdapterFieldGroup, // TAdapterControlGroup
    IAdapterDisplay, IGetAdapterDisplayCharacteristics,
    IAddFieldsEditor, IGetAdapterFormLayout)
  private
    FStyle: string;
    FCustom: string;
    FStyleRule: string;
    FLayoutAttributes: TLayoutAttributes;
//    FFormLayout: TAdapterFormLayout;
    FColumns: Integer; // added
    function FormatFields({ParentLayout: TLayout;}
      Options: TWebContentOptions): string;
    procedure SetColumns(const Value: Integer); // added
  protected
    { IAddFieldsEditor }
    function GetNewFieldPrefix: string;
    function GetAddFieldsItem: string;
    function GetAddAllFieldsItem: string;
    function GetAddFieldsDlgCaption: string;
    { IGetAdapterFormLayout }
    function GetAdapterFormLayout: TAdapterFormLayout;

    function GetLayoutAttributes: TLayoutAttributes; override;
    { IGetAdapterDisplayCharacteristics }
    function GetAdapterDisplayCharacteristics: TAdapterDisplayCharacteristics;
    { IWebComponentEditor }
    function ImplCanAddClass(AParent: TComponent; AClass: TClass): Boolean; override;
    { IWebContent }
    function ImplContent(Options: TWebContentOptions; ParentLayout: TLayout): string; override;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Style: string read FStyle write FStyle;
    property Custom: string read FCustom write FCustom;
    property StyleRule: string read FStyleRule write FStyleRule;
    property Columns: Integer read FColumns write SetColumns; // added
    property Adapter;
    property AdapterMode;
    property AddDefaultFields;
  end;

  TSimpleAdapterDisplayField = class (TAdapterDisplayField, ISimpleProduceHtml)
  private
    FColSpan: Integer;
    FCellCustom: string;
    procedure SetColSpan(const Value: Integer);
    function GetColSpan: Integer;
    procedure SetCellCustom(const Value: string);
  protected
    function GetLayoutAttributes: TLayoutAttributes; override;
    function FormatCaption: string; override;
  public
    function GetHtml (Options: TWebContentOptions): string; virtual;
    function GetCols: Integer; virtual;
  published
    property CellColSpan: Integer read GetColSpan write SetColSpan;
    property CellCustom: string read FCellCustom write SetCellCustom;
  end;

procedure Register;

implementation

{ TSimpleAdapterForm }

const
  sSetAdapterMode = '<%%'#13#10 +
                     '{'#13#10 +                            // Do not localize
                     '   var lastMode=%0:s.Mode'#13#10 +         // Do not localize
                     '   if(lastMode=="")'#13#10 +     // Do not localize
                     '   {'#13#10 +         // Do not localize
                     '     %0:s.Mode=''%1:s'''#13#10 +          // Do not localize
                     '   } %%>'#13#10 +                          // Do not localize
                     '%2:s'#13#10 +                           // Do not localize
                    '<%% %0:s.Mode=lastMode'#13#10 +         // Do not localize
                     '}  %%>'#13#10;                           // Do not localize

const
  // Params 0 - AdapterName, 1 - MemberName, 2 - Indent
  sWriteHiddenFields =
  '<%%%2:s if (%0:s.%1:s != null)'#13#10 +
   '%2:s   {'#13#10 +
   '%2:s     %0:s.%1:s.WriteFields(Response)'#13#10 +
   '%2:s   } %%>'#13#10;
  sHiddenFields = 'HiddenFields';
  sHiddenRecordFields = 'HiddenRecordFields';
  // Params 0 - AdapterName, 1 - Row
  sJSGridForEach =
  '<%% {'#13#10 +
   '   var ri = 0'#13#10 +
   '   var e = new Enumerator(%0:s.Records)'#13#10 +
   '   for (; !e.atEnd(); e.moveNext())'#13#10 +
   '   { %%>'#13#10 +
   '%1:s' +
  '<%%  ri++'#13#10 +
   '   }'#13#10 +
   '   e.moveFirst()'#13#10 +
   '   } %%>'#13#10;

function TSimpleAdapterFieldGroup.ImplContent(Options: TWebContentOptions;
  ParentLayout: TLayout): string;
begin
  Result := '<table ' + GetLayoutAttributes.ControlAttributes +
    FormatFields (Options) + '</table>';
  if (Adapter <> nil) and (AdapterMode <> '') then
    Result := Format(sSetAdapterMode,
      [MakeAdapterVariableName(Self), AdapterMode, Result]);
end;

function TSimpleAdapterFieldGroup.FormatFields(
  Options: TWebContentOptions): string;
var
  IHtml: ISimpleProduceHtml;
  I: Integer;
  VarName: string;
  colpos: integer;
begin
  colpos := 0;
  Result := '<tr>';
  for I := 0 to VisibleFields.Count - 1 do
  begin
    if Supports (IInterface(VisibleFields[I]), ISimpleProduceHtml, IHtml) then
    begin
      Result := Result + iHtml.GetHtml (Options);
      inc (colpos, ihtml.GetCols);
      // end of the row?
      if colpos >= Columns then
      begin
        colpos := 0;
        Result := Result + '</tr>' + #13#10 + '<tr>';
      end;
    end;
  end;
  Result := Result + '</tr>';

  VarName := MakeAdapterVariableName(Self);
  if VarName <> '' then
  begin
    Result := Result + Format(sWriteHiddenFields, [VarName, sHiddenFields, '']) +
    Format(sWriteHiddenFields, [VarName, sHiddenRecordFields, '']);
  end;
end;

function TSimpleAdapterFieldGroup.ImplCanAddClass(AParent: TComponent;
  AClass: TClass): Boolean;
begin
  Result := ((AClass.InheritsFrom(TAdapterDataDisplay)
    and not TAdapterDataDisplayClass(AClass).IsColumn)) or
    AClass.InheritsFrom(TCustomLayoutGroup) or
    CanAddClassHelper(Self, AParent, AClass);
end;

function TSimpleAdapterFieldGroup.GetLayoutAttributes: TLayoutAttributes;
begin
  with FLayoutAttributes do
  begin
    ControlAttributes := '';
    AddQuotedAttrib(ControlAttributes, sStyleAttr, Style);
    AddQuotedAttrib(ControlAttributes, sClassAttr, StyleRule);
    AddCustomAttrib(ControlAttributes, Custom);
  end;
  Result := FLayoutAttributes;
end;

constructor TSimpleAdapterFieldGroup.Create(AOwner: TComponent);
begin
  inherited;
  FLayoutAttributes := TLayoutAttributes.Create;
end;

destructor TSimpleAdapterFieldGroup.Destroy;
begin
  inherited;
  FLayoutAttributes.Free;
end;

function TSimpleAdapterFieldGroup.GetAdapterDisplayCharacteristics: TAdapterDisplayCharacteristics;
begin
  Result := [{dcMultipleRecordView } dcCurrentRecordView,
    dcChangeCurrentRecordView ];
end;

function TSimpleAdapterFieldGroup.GetAddAllFieldsItem: string;
begin
  Result := sAddAllFieldItems;
end;

function TSimpleAdapterFieldGroup.GetAddFieldsDlgCaption: string;
begin
  Result := sAddFieldItemsDlgCaption;
end;

function TSimpleAdapterFieldGroup.GetAddFieldsItem: string;
begin
  Result := sAddFieldItems;
end;

function TSimpleAdapterFieldGroup.GetAdapterFormLayout: TAdapterFormLayout;
begin
  Result := nil;
end;

const
  sAdapterFormItemPrefix = 'Fld'; // Do not location
function TSimpleAdapterFieldGroup.GetNewFieldPrefix: string;
begin
  Result := sAdapterFormItemPrefix;
end;

procedure Register;
begin
  RegisterWebComponents([TSimpleAdapterFieldGroup, TSimpleAdapterDisplayField]);
end;

procedure TSimpleAdapterFieldGroup.SetColumns(const Value: Integer);
begin
  FColumns := Value;
end;

{ TSimpleAdapterDisplayField }

function TSimpleAdapterDisplayField.FormatCaption: string;
var
  Attribs: string;
begin
  Attribs := '';
  AddQuotedAttrib(Attribs, sStyleAttr, CaptionAttributes.Style);
  AddCustomAttrib(Attribs, CaptionAttributes.Custom);
  AddQuotedAttrib(Attribs, sClassAttr, CaptionAttributes.StyleRule);
  if Attribs <> '' then
    Result := Format('<span%0:s>%1:s</span>', [Attribs, Caption])
  else
    Result := Caption;
end;

function TSimpleAdapterDisplayField.GetCols: Integer;
begin
  Result := CellColSpan;
end;

function TSimpleAdapterDisplayField.GetColSpan: Integer;
begin
  if fColSpan > 0 then
    Result := fColSpan
  else
    Result := 1;
end;

function TSimpleAdapterDisplayField.GetHtml (Options: TWebContentOptions): string;
begin
  // single cell for everything...
{  Result := '<td ' + GetLayoutAttributes.ControlAttributes + '>';
  case CaptionPosition of
    capLeft:
      Result := Result + FormatCaption + ' ' + ControlContent (Options);
    capRight:
      Result := Result + '<b>Attributes.LabelPosition right not supported</b>';
    capAbove:
      Result := Result + FormatCaption + ' <br> ' + ControlContent (Options);
    capBelow:
      Result := Result + '<b>Attributes.LabelPosition below not supported</b>';
  end;
  Result := Result + '</td>';}

  // double cell if left
  case CaptionPosition of
    capLeft:
      Result := '<td>' + FormatCaption + '</td><td ' +
        GetLayoutAttributes.ControlAttributes + '>' + ControlContent (Options) + '</td>';
    capRight:
      Result := '<td ' + GetLayoutAttributes.ControlAttributes + '>' +
        '<b>Attributes.LabelPosition right not supported</b></td>';
    capAbove:
      Result := '<td ' + GetLayoutAttributes.ControlAttributes + '>' +
        FormatCaption + ' <br> ' + ControlContent (Options) + '</td>';
    capBelow:
      Result := '<td ' + GetLayoutAttributes.ControlAttributes + '>' +
        '<b>Attributes.LabelPosition below not supported</b></td>';
  end;
end;

function TSimpleAdapterDisplayField.GetLayoutAttributes: TLayoutAttributes;
begin
  Result := inherited GetLayoutAttributes;

  // cell attributes
  AddQuotedAttrib(Result.ControlAttributes, 'colspan', IntToStr (FColSpan));
  AddCustomAttrib(Result.ControlAttributes, fCellCustom);

  // label attributes
  AddQuotedAttrib(Result.LabelAttributes, sVAlignAttr,
    HTMLVAlignValues[CaptionAttributes.VAlign]);
  AddQuotedAttrib(Result.LabelAttributes, sAlignAttr,
    HTMLAlignValues[CaptionAttributes.Align]);
end;

procedure TSimpleAdapterDisplayField.SetCellCustom(const Value: string);
begin
  FCellCustom := Value;
end;

procedure TSimpleAdapterDisplayField.SetColSpan(const Value: Integer);
begin
  FColSpan := Value;
end;

initialization

finalization
  UnRegisterWebComponents([TSimpleAdapterFieldGroup, TSimpleAdapterDisplayField]);

end.
